package com.fjhx.utils;

import cn.hutool.core.thread.ThreadUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class RetryUtil {

    /**
     * 重试机制
     * 默认执行3次
     * 每次失败等待1秒
     */
    public static <T> T execute(RetryRunnable<T> runnable) throws Exception {
        return execute(runnable, 0, 3, 1000L);
    }

    /**
     * 任务发生异常开启重试
     *
     * @param runnable 任务
     * @param maxCount 最大重试次数
     * @param sleep    重试间隔
     * @param <T>      返回值类型
     * @return runnable执行结果
     * @throws Exception runnable抛出的异常
     */
    public static <T> T execute(RetryRunnable<T> runnable, int maxCount, Long sleep) throws Exception {
        return execute(runnable, 0, maxCount, sleep);
    }

    private static <T> T execute(RetryRunnable<T> runnable, int executeCount, int maxCount, Long sleep) throws Exception {
        try {
            return runnable.run();
        } catch (Exception e) {
            if (executeCount >= maxCount) {
                throw e;
            }
            log.warn("任务执行失败，已失败{}次，最多允许失败{}次，{}秒后开始重试", executeCount + 1, maxCount, sleep / 1000, e);
            ThreadUtil.sleep(sleep);
            return execute(runnable, ++executeCount, maxCount, sleep);
        }
    }

    public interface RetryRunnable<T> {
        T run() throws Exception;
    }

}
